from enum import Enum

class OrderResult(str, Enum):
    """Enum for search order options."""
    RELEVANCE = "best_match"
    POPULARITY = "popularity"
    NEWEST = "date"
    UPDATED = "udate"

class ModelType(str, Enum):
    """Enum for model names used in the GameBanana API."""
    APP = "App"
    ARTICLE = "Article"
    BUG = "Bug"
    BLOG = "Blog"
    CLUB = "Club"
    CONTEST = "Contest"
    CONCEPT = "Concept"
    EVENT = "Event"
    GAME = "Game"
    IDEA = "Idea"
    INITIATIVE = "Initiative"
    JAM = "Jam"
    MOD = "Mod"
    MODEL = "Model"
    MEMBER = "Member"
    NEWS = "News"
    POLL = "Poll"
    PROJECT = "Project"
    QUESTION = "Question"
    REVIEW = "Review"
    REQUEST = "Request"
    SCRIPT = "Script"
    SOUND = "Sound"
    SPRAY = "Spray"
    STUDIO = "Studio"
    THREAD = "Thread"
    TOOL = "Tool"
    TUTORIAL = "Tutorial"
    WIKI = "Wiki"
    WIP = "Wip"